{
    "positive": {
        "required": False,
        "type": "string",
        "nullable": False,
    },
    "négative": {
        "required": False,
        "type": "string",
        "nullable": False,
    },
    "erreur": {
        "required": False,
        "type": "string",
        "nullable": False,
    },
}
